/*
 * Copyright (c) 1995, 1996, 1997, 1998 Kungliga Tekniska Hgskolan
 * (Royal Institute of Technology, Stockholm, Sweden).
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Kungliga Tekniska
 *      Hgskolan and its contributors.
 * 
 * 4. Neither the name of the Institute nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "arla_local.h"

RCSID("$Id: arladeb.c,v 1.10 1998/03/21 14:49:52 assar Exp $");

static Log_method* arla_log_metod = NULL;

struct units arla_deb_units[] = {
    { "miscellaneous",	ADEBMISC },
    { "connection",	ADEBCONN },
    { "initialization",	ADEBINIT },
    { "file-cache",	ADEBFCACHE },
    { "volume-cache",	ADEBVOLCACHE },
    { "cache-manager",	ADEBCM },
    { NULL }
};

#define DEFAULT_LOG ADEBANY

void
arla_log(unsigned level, char *fmt, ...)
{
    va_list args;
    
    if (arla_log_metod == NULL)   /* XXX Should we init here ? */
	return;
    
    va_start(args, fmt);
    log_vlog(arla_log_metod, level, fmt, args);
    va_end(args);
}

void
arla_loginit(char *log)
{
    if (log == NULL)
	log = "/dev/stderr";
    
    arla_log_metod = log_open("arla", log);
    log_set_mask(arla_log_metod, DEFAULT_LOG);
}

int
arla_log_set_level (const char *s)
{
    int ret;

    ret = parse_flags (s, arla_deb_units, 0);
    if (ret < 0)
	return ret;
    else {
	log_set_mask (arla_log_metod, ret);
	return 0;
    }
}

void
arla_log_get_level (char *s, size_t len)
{
    unparse_flags (log_get_mask (arla_log_metod),
		   arla_deb_units, s, len);
}

void
arla_log_print_levels (FILE *f)
{
    print_flags_table (arla_deb_units, f);
}

