/*
 * Copyright (c) 1995, 1996, 1997, 1998 Kungliga Tekniska Hgskolan
 * (Royal Institute of Technology, Stockholm, Sweden).
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Kungliga Tekniska
 *      Hgskolan and its contributors.
 * 
 * 4. Neither the name of the Institute nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "arla_local.h"
RCSID("$Id: irix-subr.c,v 1.6 1998/03/17 01:17:29 assar Exp $");

static long blocksize = 1024;	/* XXX */

struct args {
     int fd;
     off_t off;
     char *buf;
     char *ptr;
     struct dirent *last;
};

static void
flushbuf (struct args *args)
{
     unsigned inc = blocksize - (args->ptr - args->buf);

     args->last->d_reclen += inc;
     args->last->d_off    += inc;
     if (write (args->fd, args->buf, blocksize) != blocksize)
	  perror ("write");
     args->ptr = args->buf;
     args->last = NULL;
}

#define DIRSIZ(dp) \
        (((sizeof(struct dirent)+ (strlen((dp)->d_name)+1)) +3) & ~3)

static void
write_dirent(VenusFid *fid, const char *name, void *arg)
{
     struct dirent dirent, *real;
     struct args *args = (struct args *)arg;

/*     dirent.d_namlen = strlen (name);*/
     dirent.d_reclen = DIRSIZ(&dirent);

     if (args->ptr + dirent.d_reclen > args->buf + blocksize)
	  flushbuf (args);
     real = (struct dirent *)args->ptr;

/*     real->d_namlen = dirent.d_namlen;*/
     real->d_reclen = dirent.d_reclen;
     real->d_ino = afsfid2inode (fid);
     strcpy (real->d_name, name);
     args->ptr += real->d_reclen;
     args->off += real->d_reclen;
     real->d_off = args->off;
     args->last = real;
}

Result
conv_dir (FCacheEntry *e, char *handle, CredCacheEntry *ce, u_int tokens)
{
     struct args args;
     Result res;

     sprintf (handle, "%04X", e->inode);
     e->flags.extradirp = TRUE;
     res.tokens = e->tokens |= XFS_DATA_R | XFS_OPEN_NR;
     ReleaseSharedLock (&e->lock);
     *handle += 0x10;

     args.fd = open (handle, O_WRONLY | O_CREAT | O_TRUNC, 0666);
     if (args.fd == -1) {
	  perror ("open");
	  res.res = -1;
	  res.error = errno;
	  return res;
     }
     args.off  = 0;
     args.buf  = (char *)malloc (blocksize);
     if (args.buf == NULL)
	 errx(1, "conv_dir");
     args.ptr  = args.buf;
     args.last = NULL;
     adir_readdir (e->fid, write_dirent, (void *)&args, ce);
     if (args.last)
	  flushbuf (&args);
     free (args.buf);
     if (res.res = close (args.fd))
	  res.error = errno;
     return res;
}
