/*
 * Copyright (c) 1995, 1996, 1997 Kungliga Tekniska Hgskolan
 * (Royal Institute of Technology, Stockholm, Sweden).
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Kungliga Tekniska
 *      Hgskolan and its contributors.
 *
 * 4. Neither the name of the Institute nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/* $Id: xfs_locl.h,v 1.3 1998/03/17 02:12:06 lha Exp $ */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <sys/param.h>
#include <sys/ioctl.h>
#include <sys/filedesc.h>
#include <sys/proc.h>
#include <sys/kernel.h>
#include <sys/systm.h>
#ifdef HAVE_SYS_SYSPROTO_H
#include <sys/sysproto.h>
#endif
#include <sys/conf.h>
#include <sys/mount.h>
#include <sys/exec.h>
#ifdef HAVE_SYS_SYSENT_H
#include <sys/sysent.h>
#endif
#include <sys/lkm.h>
#include <sys/errno.h>
#include <sys/file.h>
#include <sys/namei.h>
#include <sys/dirent.h>
#include <sys/ucred.h>
#include <sys/select.h>
#ifdef HAVE_SYS_POLL_H
#include <sys/poll.h>
#endif
#include <sys/syscall.h>
#include <sys/queue.h>
#include <sys/malloc.h>


/* 
 *  The VOP table
 *
 *    What VOPs do we have today ? 
 *    This should be autoconf:ed, see it as a speed hack.... 
 */

#if 0 /* This is the full list */

#define HAVE_VOP_OPEN 1
#define HAVE_VOP_FSYNC 1
#define HAVE_VOP_CLOSE 1
#define HAVE_VOP_READ 1
#define HAVE_VOP_WRITE 1
#define HAVE_VOP_IOCTL 1
#define HAVE_VOP_SELECT 1
#define HAVE_VOP_POLL 1
#define HAVE_VOP_GETATTR 1
#define HAVE_VOP_SETATTR 1
#define HAVE_VOP_ACCESS 1
#define HAVE_VOP_LOOKUP 1
#define HAVE_VOP_CREATE 1
#define HAVE_VOP_REMOVE 1
#define HAVE_VOP_RENAME 1
#define HAVE_VOP_MKDIR 1
#define HAVE_VOP_RMDIR 1
#define HAVE_VOP_READDIR 1
#define HAVE_VOP_LINK 1
#define HAVE_VOP_SYMLINK 1
#define HAVE_VOP_READLINK 1
#define HAVE_VOP_INACTIVE 1
#define HAVE_VOP_RECLAIM 1
#define HAVE_VOP_LOCK 1
#define HAVE_VOP_UNLOCK 1
#define HAVE_VOP_ABORTOP 1
#define HAVE_VOP_MMAP 1
#define HAVE_VOP_BMAP 1
#define HAVE_VOP_GETPAGES 1
#define HAVE_VOP_PUTPAGES 1
#define HAVE_VOP_CMP 1
#define HAVE_VOP_REALVP 1
#define HAVE_VOP_CNTL

#endif


/*
 * Common parts for Net/Open/Free-BSD
 */

#if defined(__NetBSD__) || defined(__OpenBSD__) || defined(__FreeBSD__)
#define HAVE_VOP_OPEN 1
#define HAVE_VOP_FSYNC 1
#define HAVE_VOP_CLOSE 1
#define HAVE_VOP_READ 1
#define HAVE_VOP_WRITE 1
#define HAVE_VOP_IOCTL 1
#define HAVE_VOP_GETATTR 1
#define HAVE_VOP_SETATTR 1
#define HAVE_VOP_ACCESS 1
#define HAVE_VOP_LOOKUP 1
#define HAVE_VOP_CREATE 1
#define HAVE_VOP_REMOVE 1
#define HAVE_VOP_RENAME 1
#define HAVE_VOP_MKDIR 1
#define HAVE_VOP_RMDIR 1
#define HAVE_VOP_READDIR 1
#define HAVE_VOP_LINK 1
#define HAVE_VOP_SYMLINK 1
#define HAVE_VOP_READLINK 1
#define HAVE_VOP_INACTIVE 1
#define HAVE_VOP_RECLAIM 1
#define HAVE_VOP_LOCK 1
#define HAVE_VOP_UNLOCK 1
#define HAVE_VOP_ABORTOP 1
#define HAVE_VOP_MMAP 1
#define HAVE_VOP_BMAP 1

#endif /* Common parts */


/*
 * Now fix all the broken things for the *BSD os...
 */

#if defined(__NetBSD__)

#define HAVE_VOP_POLL 1

#elif defined(__OpenBSD__)

#define HAVE_VOP_SELECT 1

#elif defined(__FreeBSD__)

#define HAVE_VOP_GETPAGES 1
#define HAVE_VOP_PUTPAGES 1
#define HAVE_VOP_SELECT 1

#else
#error do not know your bsd 
#endif
